;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Raccoon guy
;;
;; Description: This sprite follows the player and jumps when the player jumps
;;
;; Uses first extra bit: NO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!RAM_FRAME		= $1528
	!RAM_DIRECTION	= $151C

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                   
PROPERTIES:	db $0F,$0F,$0F,$0F	; xyppccct format
WALKING1:	db $80,$91,$82,$A0
WALKING2:	db $83,$94,$85,$A3
WALKING3:	db $B0,$C1,$B2,$D0
JUMPING2:	db $B3,$C4,$B5,$D3
TILEPOSX:	db $FC,$04,$0B,$FC
TILEPOSX1:	db $04,$FC,$FC,$0C
TILEPOSY:	db $F8,$00,$F8,$08
TILESIZE:	db $02,$02,$00,$00
ANIMATION:	db $00,$04,$08,$04,$0C
;				1	2	3	4	F  (1-4 Walking animation, F flying)
;				>   <
XSPEED:		db $08,$F8

!JUMPSPEED	= $C0
!FALL_SPEED	= $08

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
		JSR CheckDirection
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "MAIN ",pc
		PHB	         			; \ 
		PHK                     ;  | main sprite function, just calls local subroutine
		PLB                     ;  |
		JSR SPRITE_CODE_START   ;  |
		PLB                     ;  |
	RTL                     	; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main sprite routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPRITE_CODE_START:
		LDA $AA,x
		BMI NOT_FALLING
		CMP #$01
		BCC NOT_FALLING
		LDA #$04
		STA !RAM_FRAME,x
		BRA GFX
NOT_FALLING:
		LDA !RAM_FRAME,x
		CMP #$04
		BNE GFX
		STZ !RAM_FRAME,x
GFX:		JSR SUB_GFX
		LDY #$00
		JSR SubOffScreen 	; Handle sprite off-screen
		LDA $9D
		BNE RETURN0
		LDA $14C8,x
		CMP #$08
		BNE RETURN0
		LDA $14 : AND #$3F
		BNE NO_DIR_CHECK
		JSR CheckDirection
NO_DIR_CHECK:
		LDY !RAM_DIRECTION,x
		LDA XSPEED,y
		STA $B6,x
		JSL $01802A ; > Update Sprite's X/Y position with gravity
		LDA $AA,x
		BMI SKIP
		LDA #!FALL_SPEED
		STA $AA,x
SKIP:	JSL $019138 ; > Check collision with objects
		JSL $01A7DC ; > Sprite <-> Player interaction routine
		LDA $1588,x
		AND #$03
		BEQ NO_DIR_CHECK1
		LDA !RAM_DIRECTION,x
		EOR #$01
		STA !RAM_DIRECTION,x
NO_DIR_CHECK1:
		LDA $1588,x
		AND #$08
		BEQ NO_CEILING
		LDA #!FALL_SPEED
		STA $AA,x
NO_CEILING:
		LDA $1588,x
		AND #$04
		BEQ NOT_ON_GROUND
		STZ $AA,x ; Reset gravity
		LDA $16
		ORA $18
		AND #$80
		BEQ NOT_ON_GROUND
		LDA $77 ; > Player blocked status
		AND #$04
		BEQ NOT_ON_GROUND
		LDA #!JUMPSPEED
		STA $AA,x
NOT_ON_GROUND:
RETURN0:
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GFX:
		JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
			;      $00 = sprite x position relative to screen boarder 
            ;      $01 = sprite y position relative to screen boarder
			
		LDA !RAM_DIRECTION,x
		ROR : ROR : LSR
		EOR #$40
		STA $0E
		
		STY $0F
		LDY #$03
		LDA !RAM_DIRECTION,x
		BEQ LOOP2
LOOP1:	LDA TILEPOSX,y
		STA $0002,y
		DEY : BPL LOOP1
		BRA DONE
LOOP2:	LDA TILEPOSX1,y
		STA $0002,y
		DEY : BPL LOOP2
DONE:	LDY $0F
		LDA !RAM_FRAME,x
		CMP #$04
		BEQ FLYING
		LDA $14 : AND #$07
		BNE FLYING
		LDA !RAM_FRAME,x
		INC : AND #$03
		STA !RAM_FRAME,x
FLYING:	LDX #$03 ; loop counter
GFX_LOOP:
		LDA $00
		CLC : ADC $02,x
		STA $0300,y
		LDA $01
		CLC : ADC TILEPOSY,x
		STA $0301,y
		
		LDA PROPERTIES,x
		ORA $64
		EOR $0E
		STA $0303,y
		
		STY $0F
		TYA : LSR : LSR : TAY
		LDA TILESIZE,x
		STA $0460,y
		LDY $0F
		
		STX $0F
		LDX $15E9
		LDA !RAM_FRAME,x
		TAX
		LDA $0F
		CLC : ADC ANIMATION,x
		TAX
		LDA WALKING1,x
		STA $0302,y
		LDX $0F
		
		INY : INY ; \ Next OAM slot
		INY : INY ; /
		DEX : BPL GFX_LOOP	; > check if loop is done and repeat if necessary
		
		LDX $15E9	; > Get currently processed sprite index back into X
		
		LDY #$FF				; > FF because the size is already set
		LDA #$03                ; \ A = number of tiles drawn - 1
		JSL $01B7B3             ; /
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CheckDirection:
	LDA $14E0,x		; \ Load sprite x position (16-bit) into A
	XBA				; |
	LDA $E4,x		; /
	REP #$20		; \ Subtract players x position so that carry set = player to the right of the sprite
	SEC : SBC $94	; |
	SEP #$20		; /
	PHP : PLA			; \ Get processor flags into A
	ROL : AND #$01		; | Get carry into first bit of A and remove all other bits
	STA !RAM_DIRECTION,x; /
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1:              db $0C,$1C
TABLE2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \ 
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

                    ;org $03B83B             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
SubOffScreen:	LDA $15A0,x
		ORA $186C,x
		BEQ .Return2
		LDA $167A,x
		AND #$04
		BNE .Return2
		LDA $5B
		LSR A
		BCS .VerticalLevel
		LDA $D8,x
		ADC #$50
		LDA $14D4,x
		ADC #$00
		CMP #$02
		BCS .EraseSprite
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC .AddTable,y
		SEC
		SBC $1A
		CMP .CmpTable,y
.Common		SEP #$20
		BCC .Return2
.EraseSprite	LDA $14C8,x
		CMP #$08
		BCC .KillSprite
		LDY $161A,x
		CPY #$FF
		BEQ .KillSprite
		LDA $1938,y
		AND #$FE
		STA $1938,y
.KillSprite	STZ $14C8,x
.Return2	RTS

.VerticalLevel	LDA $13
		LSR A
		BCS .CheckY
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC.w #$0040
		CMP.w #$0280
		BRA .Common
.CheckY		LDA $14D4,x
		XBA
		LDA $D8,x
		REP #$20
		SBC $1C
		CLC
		ADC.w #$0070
		CMP.w #$01D0
		BRA .Common

.AddTable	dw $0040,$0040,$0010,$0070
		dw $0090,$0050,$0080,$FFC0
.CmpTable	dw $0170,$01E0,$01B0,$01D0
		dw $0230,$01B0,$0220,$0160
		